import { QueryKey, UseQueryOptions, UseMutationOptions, UseMutationResult, UseQueryResult } from '@tanstack/react-query';
import * as react from 'react';

type ServiceMethod<I, O> = (req: I, initReq: RequestInitWithPathPrefix) => Promise<O>;
interface RequestInitWithPathPrefix extends RequestInit {
    pathPrefix?: string;
}
type ServiceError = Error | ErrorResponse;
type OnErrorHandler<O> = (error: Error | ErrorResponse) => O | null;
interface ErrorResponse {
    codeName?: string;
    code: number;
    message: string;
    details: unknown[];
}
type UseServiceQueryOptions<M extends ServiceMethod<Parameters<M>[0], ReturnType<M>>, Q extends QueryKey> = Omit<UseQueryOptions<Awaited<ReturnType<M>>, ServiceError>, 'queryFn' | 'queryKey'> & {
    onError?: OnErrorHandler<ReturnType<M>>;
} & {
    queryKey?: Q;
};
type UseServiceMutationOptions<M extends ServiceMethod<Parameters<M>[0], ReturnType<M>>, C = unknown> = Omit<UseMutationOptions<Awaited<ReturnType<M>>, ServiceError, Parameters<M>[0], C>, 'mutationFn'>;
declare function isErrorResponse(error: Error | ErrorResponse): error is ErrorResponse;

declare function useServiceMutation<M extends ServiceMethod<Parameters<M>[0], Awaited<ReturnType<M>>>, C = unknown>(method: M, options?: UseServiceMutationOptions<M, C>): UseMutationResult<Awaited<ReturnType<M>>, ServiceError, Partial<Parameters<M>[0]>, C>;

declare function useServiceQuery<M extends ServiceMethod<Parameters<M>[0], Awaited<ReturnType<M>>>, Q extends QueryKey>(method: M, req: Parameters<M>[0], options?: UseServiceQueryOptions<M, Q>): UseQueryResult<Awaited<ReturnType<M>>, ServiceError>;
declare function queryOptions<M extends ServiceMethod<Parameters<M>[0], Awaited<ReturnType<M>>>, Q extends QueryKey>(method: M, req: Parameters<M>[0], reqInit?: RequestInitWithPathPrefix, options?: UseServiceQueryOptions<M, Q>): UseQueryOptions<Awaited<ReturnType<M>>, ServiceError>;

declare const ServiceContext: react.Context<RequestInitWithPathPrefix>;
declare const useServiceContext: () => RequestInitWithPathPrefix;

declare function queryKey<M extends ServiceMethod<Parameters<M>[0], Awaited<ReturnType<M>>>>(method: M, req?: Parameters<M>[0]): [string] | [string, Parameters<M>[0]];

export { type ErrorResponse, type OnErrorHandler, type RequestInitWithPathPrefix, ServiceContext, type ServiceError, type ServiceMethod, type UseServiceMutationOptions, type UseServiceQueryOptions, isErrorResponse, queryKey, queryOptions, useServiceContext, useServiceMutation, useServiceQuery };
