// src/types.ts
function isErrorResponse(error) {
  return error.code !== void 0;
}

// src/useServiceMutation.ts
import { useMutation } from "@tanstack/react-query";
import { useContext } from "react";
function useServiceMutation(method, options) {
  const reqCtx = useContext(ServiceContext);
  return useMutation({
    ...options,
    mutationFn: (req) => {
      const resp = method(req, {
        ...reqCtx,
        headers: {
          "Content-Type": "application/json",
          ...reqCtx.headers
        }
      });
      return resp;
    }
  });
}

// src/useServiceQuery.ts
import { useQuery } from "@tanstack/react-query";
import { useContext as useContext2 } from "react";

// src/queryKey.ts
function queryKey(method, req) {
  if (req === void 0)
    return [method.name];
  return [method.name, req];
}

// src/useServiceQuery.ts
function useServiceQuery(method, req, options) {
  const reqCtx = useContext2(ServiceContext);
  return useQuery(queryOptions(method, req, reqCtx, options));
}
function queryOptions(method, req, reqInit, options) {
  return {
    ...options,
    queryKey: options?.queryKey ?? queryKey(method, req),
    queryFn: () => {
      const resp = method(req, {
        ...reqInit,
        headers: {
          "Content-Type": "application/json",
          ...reqInit?.headers
        }
      });
      if (options?.onError) {
        return resp.catch(options.onError);
      }
      return resp;
    }
  };
}

// src/serviceContext.ts
import { createContext, useContext as useContext3 } from "react";
var ServiceContext = createContext({});
var useServiceContext = () => {
  return useContext3(ServiceContext);
};
export {
  ServiceContext,
  isErrorResponse,
  queryKey,
  queryOptions,
  useServiceContext,
  useServiceMutation,
  useServiceQuery
};
//# sourceMappingURL=index.js.map